/* Nessus
 * Copyright (c) mjh-EDV Beratung, 1999
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 */

#ifndef __TASK_MAKER_H__
#define __TASK_MAKER_H__

/* ---------------------------------------------------------------------- *
 *                  definitions and data structures                       *
 * ---------------------------------------------------------------------- */

/* generic variant selector, data object label and message type */
typedef
enum _taskobj {

  IS_NOTHING = 0,

  /* some task types to be mapped to some explicit communication and
     sub-process method, below */
  START_A_THREAD, FORK_A_CHILD,
  
  /* explicitly defined communication channels and task types */
  ISA_PIPE_CHILD_MASTER, ISA_PIPE_THREAD_MASTER, ISA_SHMEM_THREAD_MASTER,

  /* slave parts using inherited defs */
  ISA_PIPE_CHILD_SLAVE, ISA_PIPE_THREAD_SLAVE, ISA_SHMEM_THREAD_SLAVE,

  /* we might route through the master entry */
  ISA_PIPE_ROUTER_SLAVE,

  /* message labels */
  MSG_PING, MSG_STOP, MSG_ACK, MSG_NAK, MSG_ERRNO, MSG_THREADID,
  
  LAST_ENTRY_UNUSED
} taskobj ;

/* generic messages */
typedef
struct _mesg {
  int size ;		/* total size of this record */
  struct _mesg* next;	/* messages can be queued */
  int src, trg ;	/* sender and receiver id */
  unsigned msgid ;	/* current message id */
  unsigned ackid ;	/* the message is the answer to something */
  taskobj mesg_type ;	/* quick messages or type of data, following */
  char data [1] ;	/* message text */
  /* variable length */
} mesg ;

/* communication channel between master and task */
struct _taskdef ;

/* single task descriptor */
typedef
struct _taskrec {
  int taskid, msgid ;

# ifdef ENABLE_CRYPTO_LAYER
  /* common write descriptor, etc. */
  int soc, tid ;
# endif

  /* the task function to be invoked */
  void (*fn)(struct _taskdef*, void*);
  void *fn_arg ;	/* opaque function argument pointer */

  /* task type and communication defs */
  taskobj task_type ;
  union {		/* task types */
    struct { pid_t pid ; }  child ;
#   ifdef USE_PTHREADS
    struct { pthread_t tid ; } thread ;
#   endif
  } tt ;
  union {		/* communication channels */
    struct { int recv, send ; } pipe ;
    struct { struct _taskdef *task; } master ;
    struct { struct _taskdef *task; } slave ;
  } io ;
} taskrec ;


typedef
struct _taskdef {

  /* set this if (tasklist [id].teskid == id) need not be true
     in that case, the sparse list has the latest id, available */
  unsigned sparse_list ;

  mesg *incoming ;	/* incoming messages */

# ifdef HAVE_SELECT	/* waiting for a read data event */
  fd_set         read_selector ;
  unsigned limit_read_selector ;
# endif
# ifdef HAVE_POLL	/* waiting for a read data event */
  struct pollfd *read_poller ;
  unsigned   dim_read_poller ;
# endif
  int   shared_count;	/* read polling */
# ifdef USE_PTHREADS
  pthread_mutex_t mx;	/* shared access with pthreads */
# endif

  /* give the threads a chance to signal an alarm */
  int waiting_for_some_time ;

  unsigned act_task  ;	/* the index of the current task */
  unsigned dim_tasks ;	/* number of task entries, following */
  taskrec     *tasks ;	/* the first entry is the self descriptor */

} taskdef ;


/* stdout, implements sleep () using select/poll */
#ifndef FILE_LOCK_POLL_FD
#define FILE_LOCK_POLL_FD      1
#endif

/* ---------------------------------------------------------------------- *
 *                         functional interface                           *
 * ---------------------------------------------------------------------- */

#endif /* __TASK_MAKER_H__ */
